<?php
class ControllerExtensionPaymentPaynet extends Controller {

  public function index() {
    $this->language->load('extension/payment/paynet');
    $this->load->model('checkout/order');
    $this->load->model('catalog/category');
    $this->load->model('catalog/product');
    date_default_timezone_set('Europe/Chisinau');
    /* Paynet works only with MDL, so check if MDL is enabled on the site */
    if($this->currency->has('MDL')){
      $data['mdl_enabled'] = true;
      $testMode = intval($this->config->get('payment_paynet_test_mode'));
      if($testMode == 1){
        $data['action'] = 'https://test.paynet.md/Acquiring/SetEcom';
      } else {
        $data['action'] = 'https://paynet.md/Acquiring/SetEcom';
      }

      $order_id = $this->session->data['order_id'];

      $order_info = $this->model_checkout_order->getOrder($order_id);
      $products_info = $this->cart->getProducts();
      $shipping_info = $this->session->data['shipping_method'];

      if(!empty($order_info) && is_array($order_info)){
        $data['order_id'] = $order_id.$this->generateIdPostfix();
        $data["merchant"] = $this->config->get('payment_paynet_merchant_code');
        $data["currency"] = '498'; //$order_info['currency_code'];
        $data["service_name"] = $_SERVER['HTTP_HOST'];
        $data["service_description"] = $this->language->get('service_description');
        /* Allways convert amount to MDL */
        $data["amount"] = round($this->currency->format($order_info['total'], 'MDL' , false, false), 2) * 100;
        /* Prepare products */
        $products = [];
        $i=0;
        foreach ($products_info as $item) {
          $categories = $this->model_catalog_product->getCategories($item['product_id']);
          if($categories){
            $categories_info = $this->model_catalog_category->getCategory($categories[0]['category_id']); 
          }
          $product = [];
          $product['Amount'] = round($this->currency->format($item['price'] * $item['quantity'], 'MDL' , false, false), 2) * 100;
          $product['Barcode'] = $item['product_id'];
          $product['Code'] = $item['product_id'];
          $product['Description'] = $item['name'];
          $product['GroupId'] = $categories_info['category_id'];
          $product['GroupName'] = $categories_info['name'];
          $product['LineNo'] = $i;
          $product['Name'] = $item['name'];
          $product['Quantity'] =  $item['quantity']; // Exclude from signature generation
          $product['UnitPrice'] = round($this->currency->format($item['price'], 'MDL' , false, false), 2) * 100;
          $product['UnitProduct'] = 'PCS';
          array_push($products, $product);
          $i++;
        }

        // Add shipping data to prevent errors caused by different ammount and items total
        $product = [];
        $product['Amount'] = round($this->currency->format($shipping_info['cost'], 'MDL' , false, false), 2) * 100;
        $product['Barcode'] = '101010101'; // dummy data
        $product['Code'] = 'shipping';
        $product['Description'] = $shipping_info['title'];
        $product['GroupId'] = $shipping_info['tax_class_id'];
        $product['GroupName'] = 'Shipping';
        $product['LineNo'] = $i;
        $product['Name'] = $shipping_info['title'];
        $product['Quantity'] =  1; // Exclude from signature generation
        $product['UnitPrice'] = round($this->currency->format($shipping_info['cost'], 'MDL' , false, false), 2) * 100;
        $product['UnitProduct'] = 'PCS';
        array_push($products, $product);

        $data['products'] = $products;

        // Prepare customer data
        $customer = [];
        $customer['Code'] = $order_info['customer_id'];
        $customer['NameFirst'] = html_entity_decode($order_info['shipping_firstname'], ENT_QUOTES, 'UTF-8');
        $customer['NameLast'] = html_entity_decode($order_info['shipping_lastname'], ENT_QUOTES, 'UTF-8');
        $customer['PhoneNumber'] = html_entity_decode($order_info['telephone'], ENT_QUOTES, 'UTF-8');
        $customer['email'] = $order_info['email'];
        $customer['Country'] = html_entity_decode($order_info['payment_country'], ENT_QUOTES, 'UTF-8');
        $customer['City'] = html_entity_decode($order_info['payment_city'], ENT_QUOTES, 'UTF-8');
        $customer['Address'] = html_entity_decode($order_info['payment_address_1'], ENT_QUOTES, 'UTF-8');

        $data['customer'] = $customer;

        $data["external_date"] = date('Y-m-d', time()).'T'.date('H:i:s', time());
        $date = strtotime("+2 hour");
        $data["expiry_date"] = date('Y-m-d', $date).'T'.date('H:i:s', $date);
        $data["url_success"] = $this->url->link('extension/payment/paynet/success','&order_id='.$this->session->data['order_id'], 'SSL');
        $data["url_cancel"] = $this->url->link('checkout/checkout', '', 'SSL');
        $data["lang"] = $this->_language($order_info['language_code']);
        $data["api_version"] = 'v05';
        $data["money_type"] = 'VISA_AND_MASTER_MDL';

        $PreparedString = $data["currency"] . $customer["Address"] . $customer["City"] . $customer["Code"] . $customer["Country"] . $customer["email"] . $customer["NameFirst"] . $customer["NameLast"] . $customer["PhoneNumber"] . $data["expiry_date"] . strval($data["order_id"]) . $data["merchant"] . $data["money_type"] . $data["amount"] . $data["service_description"] . $data["service_name"];
        foreach ($products as $product) {
          foreach ($product as $key => $value) {
            if($key != 'Quantity'){
              $PreparedString.=$value;
            }
          }
        }
        $PreparedString .= trim($this->config->get('payment_paynet_secret_key'));
        $Signature = base64_encode(md5($PreparedString,true));
        $data["signature"] = $Signature;

      } else {
        $this->log->write("Can not obtain order info for order ".$order_id);
        return false;
      }

      /* Get awailable methods */
      $methods = $this->config->get('payment_paynet_payment_method');
      $pm = [
        'paynet_method' => [
          'code' => 'PAYNET',
          'label' => 'Paynet'
        ],
        'vmmdl' => [
          'code' => 'VISA_AND_MASTER_MDL',
          'label' => 'Visa/Mastercard (MDL)'
        ],
        'vmusd' => [
          'code' => 'VISA_AND_MASTER_USD',
          'label' => 'Visa/Mastercard (USD)'
        ],
        'vmeur' => [
          'code' => 'VISA_AND_MASTER_EUR',
          'label' => 'Visa/Mastercard (EUR)'
        ],
        'qiwi' => [
          'code' => 'QIWI',
          'label' => 'Qiwi'
        ],
        'yandex' => [
          'code' => 'YANDEX',
          'label' => 'Yandex'
        ],
        'btc' => [
          'code' => 'BTC',
          'label' => 'Yandex'
        ]
      ];

      $available = [];
      foreach ($methods as $key => $status) {
        if($status){
          $available[] = $pm[$key];
        }
      }

      $data['available_methods'] = $available;
      
      $data['button_confirm'] = $this->language->get('button_confirm');

    } else {
      $data['mdl_enabled'] = false;
      $data['currency_error'] = 'MDL currency is not enabled on the site! Please make a step back and use another payment gateway!';
    }
   
    return $this->load->view('extension/payment/paynet', $data);
  }

  public function generateIdPostfix(){
  	/* 	Generate a random string wich will be added to order id, 
  		to prevent errors when order with same id is redirected 
  		to paynet more than one time
  	*/
  	return substr(number_format(microtime(true), 4, '', ''), -10);
  }

  public function getPaynetApiAuthToken($api_url, $api_login, $api_password){
      	$ch = curl_init($api_url . '/auth'); 
        header('Content-Type: application/json');
        $data = 'grant_type=password&username=' . $api_login . '&password=' . $api_password; 
        curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type:application/x-www-form-urlencoded'));
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS,$data);
        $response_raw = curl_exec($ch);
        curl_close($ch);
       
    		$this->log_transaction('Auth token request', $response_raw);

        $response = json_decode($response_raw,true);
        $token = $response['access_token'];
        return $token;
  }

  public function checkPaynetPaymentStatus($api_url, $token, $payment_id){
	    $ch = curl_init($api_url . '/api/Payments/' . $payment_id);
	    header('Content-Type: application/json');
	    $authorization = "Authorization: Bearer ".$token; // **Prepare Autorisation Token**
	    curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json' , $authorization )); // **Inject Token into Header**
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);   
	    $response_raw = curl_exec($ch);
	    curl_close($ch);
	    $this->log_transaction('Check payment status request', $response_raw);
	    $response = json_decode($response_raw,true);
	    return $response['Status'];
  }

  public function success() {

    if (isset($this->session->data['order_id'])) {
      $order_id = $this->session->data['order_id'];
    } else {
      $order_id = 0;
    }

    $this->load->model('checkout/order');
    $order_info = $this->model_checkout_order->getOrder($order_id);

    $status = $order_info['order_status_id'];
    if($status === $this->config->get('payment_paynet_completed_status_id')){
      $this->response->redirect($this->url->link('checkout/success', '', 'SSL'));
    } else {
      echo $this->load->view('extension/payment/paynet_waiting');
    }
  }

  public function callback() {
  	// Read incoming info
    $response_raw =  (string)file_get_contents("php://input");
    if(!empty($response_raw)){
      $this->log_transaction('Notification', $response_raw);
    	$response = json_decode($response_raw, true);
    	$external_id = $response['Payment']['ExternalId']; //order id with generated postfix
    	$order_id = substr($external_id, 0, -10); //remove last 10 digits of postfix
	    $payment_id = $response['Payment']['Id']; //payment id on paynet

	    $testMode = intval($this->config->get('payment_paynet_test_mode'));
      if($testMode == 1){
        $api_url = 'https://test.paynet.md:4446';
      } else {
        $api_url = 'https://paynet.md:4448';
      }
	    $api_login = $this->config->get('payment_paynet_api_login');
	    $api_password = urlencode(html_entity_decode($this->config->get('payment_paynet_api_password')));

	    if ($response['EventType'] === 'PAID') {
		        $token = $this->getPaynetApiAuthToken($api_url, $api_login, $api_password);
            if($token){
              $status = $this->checkPaynetPaymentStatus($api_url, $token, $payment_id);
            }

            $this->load->model('checkout/order');
			      $order_info = $this->model_checkout_order->getOrder($order_id);
			      if ($order_info) {
			        $this->model_checkout_order->addOrderHistory($order_id, $this->config->get('config_order_status_id'));
              $this->log_transaction('Status changed', $this->config->get('config_order_status_id'));
			      }
            if(isset($status) && $status === 4){
			        
  			      $this->model_checkout_order->addOrderHistory($order_id, $this->config->get('payment_paynet_completed_status_id'), "Transaction ID: $payment_id", true);
              $this->log_transaction('Status changed', $this->config->get('payment_paynet_completed_status_id'));

  			      die('OK');
			  
            }
	     }
    } else {
      var_dump($this->config->get('payment_paynet_payment_method'));
    		$this->log_transaction('Callback accessed', 'No incoming data detected!');
    		die('Illegal Access!');
    }
  }

  private function _language($lang_id) {
    $lang = substr($lang_id, 0, 2);
    $lang = strtolower($lang);
    return $lang;
  }

  protected function log_transaction($title, $response){
      date_default_timezone_set('Europe/Chisinau');
      $log_file_url = $_SERVER['DOCUMENT_ROOT'].'/system/logs/paynet.log';
      $date=date("m.d.y");
      $time=date('H:i:s', time());
      $data = "--------------------------------------------------------------------------------\nAction: $title\nDate/Time: $date / $time \nResponse: $response\n";
      file_put_contents($log_file_url, $data, FILE_APPEND | LOCK_EX);
  }
}
